@@ -48,6 +48,7 @@ get_kaslr_offset_x86_64(unsigned long vaddr)
 {
 	unsigned int i;
 	char buf[BUFSIZE_FGETS], *endp;
+	unsigned long kernel_image_size;
 
 	if (!info->kaslr_offset && info->file_vmcoreinfo) {
 		if (fseek(info->file_vmcoreinfo, 0, SEEK_SET) < 0) {
@@ -68,14 +69,22 @@ get_kaslr_offset_x86_64(unsigned long vaddr)
 					strtoul(buf+strlen(STR_KERNELOFFSET),&endp,16);
 		}
 	}
+	if (!info->kaslr_offset || !vaddr)
+		return 0;
+
+	if (NUMBER(KERNEL_IMAGE_SIZE) != NOT_FOUND_NUMBER)
+		kernel_image_size = NUMBER(KERNEL_IMAGE_SIZE);
+	else
+		kernel_image_size = KERNEL_IMAGE_SIZE_KASLR_ORIG;
+
+	/*
+	 * Returns the kaslr offset only if the vaddr needs it to be added,
+	 * i.e. only kernel text address for now.  Otherwise returns 0.
+	 */
 	if (vaddr >= __START_KERNEL_map &&
-			vaddr < __START_KERNEL_map + info->kaslr_offset)
+			vaddr < __START_KERNEL_map + kernel_image_size)
 		return info->kaslr_offset;
 	else
-		/*
-		 * TODO: we need to check if it is vmalloc/vmmemmap/module
-		 * address, we will have different offset
-		 */
 		return 0;
 }
 
@@ -149,6 +158,8 @@ get_phys_base_x86_64(void)
 	info->phys_base = 0; /* default/traditional */
 	if (NUMBER(phys_base) != NOT_FOUND_NUMBER) {
 		info->phys_base = NUMBER(phys_base);
+		DEBUG_MSG("phys_base    : %lx (vmcoreinfo)\n",
+			info->phys_base);
 		return TRUE;
 	}
 
@@ -156,6 +167,8 @@ get_phys_base_x86_64(void)
 	if (!has_vmcoreinfo() && info->name_vmlinux) {
 		SYMBOL_INIT(phys_base, "phys_base");
 		if (SYMBOL(phys_base) == NOT_FOUND_SYMBOL) {
+			DEBUG_MSG("phys_base    : %lx (no phys_base)\n",
+				info->phys_base);
 			return TRUE;
 		}
 	}
@@ -167,6 +180,8 @@ get_phys_base_x86_64(void)
 			info->phys_base = phys_start -
 			    (virt_start & ~(__START_KERNEL_map));
 
+			DEBUG_MSG("phys_base    : %lx (pt_load)\n",
+				info->phys_base);
 			break;
 		}
 	}
@@ -253,10 +268,10 @@ get_machdep_info_x86_64(void)
 int
 get_versiondep_info_x86_64(void)
 {
-	/*
-	 * On linux-2.6.26, MAX_PHYSMEM_BITS is changed to 44 from 40.
-	 */
-	if (info->kernel_version < KERNEL_VERSION(2, 6, 26))
+	/* Check if we can get MAX_PHYSMEM_BITS from vmcoreinfo */
+	if (NUMBER(MAX_PHYSMEM_BITS) != NOT_FOUND_NUMBER)
+		info->max_physmem_bits = NUMBER(MAX_PHYSMEM_BITS);
+	else if (info->kernel_version < KERNEL_VERSION(2, 6, 26))
 		info->max_physmem_bits  = _MAX_PHYSMEM_BITS_ORIG;
 	else if (info->kernel_version < KERNEL_VERSION(2, 6, 31))
 		info->max_physmem_bits  = _MAX_PHYSMEM_BITS_2_6_26;
@@ -291,6 +306,7 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 	unsigned long page_dir, pgd, pud_paddr, pud_pte, pmd_paddr, pmd_pte;
 	unsigned long pte_paddr, pte;
 	unsigned long p4d_paddr, p4d_pte;
+	unsigned long entry_mask = ENTRY_MASK;
 
 	/*
 	 * Get PGD.
@@ -302,6 +318,9 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 			return NOT_PADDR;
 	}
 
+	if (NUMBER(sme_mask) != NOT_FOUND_NUMBER)
+		entry_mask &= ~(NUMBER(sme_mask));
+
 	if (check_5level_paging()) {
 		page_dir += pgd5_index(vaddr) * sizeof(unsigned long);
 		if (!readmem(PADDR, page_dir, &pgd, sizeof pgd)) {
@@ -318,7 +337,7 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 		/*
 		 * Get P4D.
 		 */
-		p4d_paddr  = pgd & ENTRY_MASK;
+		p4d_paddr = pgd & entry_mask;
 		p4d_paddr += p4d_index(vaddr) * sizeof(unsigned long);
 		if (!readmem(PADDR, p4d_paddr, &p4d_pte, sizeof p4d_pte)) {
 			ERRMSG("Can't get p4d_pte (p4d_paddr:%lx).\n", p4d_paddr);
@@ -331,7 +350,7 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 			ERRMSG("Can't get a valid p4d_pte.\n");
 			return NOT_PADDR;
 		}
-		pud_paddr  = p4d_pte & ENTRY_MASK;
+		pud_paddr = p4d_pte & entry_mask;
 	}else {
 		page_dir += pgd_index(vaddr) * sizeof(unsigned long);
 		if (!readmem(PADDR, page_dir, &pgd, sizeof pgd)) {
@@ -345,7 +364,7 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 			ERRMSG("Can't get a valid pgd.\n");
 			return NOT_PADDR;
 		}
-		pud_paddr  = pgd & ENTRY_MASK;
+		pud_paddr = pgd & entry_mask;
 	}
 
 	/*
@@ -364,13 +383,13 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 		return NOT_PADDR;
 	}
 	if (pud_pte & _PAGE_PSE)	/* 1GB pages */
-		return (pud_pte & ENTRY_MASK & PUD_MASK) +
+		return (pud_pte & entry_mask & PUD_MASK) +
 			(vaddr & ~PUD_MASK);
 
 	/*
 	 * Get PMD.
 	 */
-	pmd_paddr  = pud_pte & ENTRY_MASK;
+	pmd_paddr = pud_pte & entry_mask;
 	pmd_paddr += pmd_index(vaddr) * sizeof(unsigned long);
 	if (!readmem(PADDR, pmd_paddr, &pmd_pte, sizeof pmd_pte)) {
 		ERRMSG("Can't get pmd_pte (pmd_paddr:%lx).\n", pmd_paddr);
@@ -384,13 +403,13 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 		return NOT_PADDR;
 	}
 	if (pmd_pte & _PAGE_PSE)	/* 2MB pages */
-		return (pmd_pte & ENTRY_MASK & PMD_MASK) +
+		return (pmd_pte & entry_mask & PMD_MASK) +
 			(vaddr & ~PMD_MASK);
 
 	/*
 	 * Get PTE.
 	 */
-	pte_paddr  = pmd_pte & ENTRY_MASK;
+	pte_paddr = pmd_pte & entry_mask;
 	pte_paddr += pte_index(vaddr) * sizeof(unsigned long);
 	if (!readmem(PADDR, pte_paddr, &pte, sizeof pte)) {
 		ERRMSG("Can't get pte (pte_paddr:%lx).\n", pte_paddr);
@@ -403,7 +422,7 @@ __vtop4_x86_64(unsigned long vaddr, unsigned long pagetable)
 		ERRMSG("Can't get a valid pte.\n");
 		return NOT_PADDR;
 	}
-	return (pte & ENTRY_MASK) + PAGEOFFSET(vaddr);
+	return (pte & entry_mask) + PAGEOFFSET(vaddr);
 }
 
 unsigned long long
@@ -636,6 +655,7 @@ find_vmemmap_x86_64()
 	unsigned long pmd, tpfn;
 	unsigned long pvaddr = 0;
 	unsigned long data_addr = 0, last_data_addr = 0, start_data_addr = 0;
+	unsigned long pmask = PMASK;
 	/*
 	 * data_addr is the paddr of the page holding the page structs.
 	 * We keep lists of contiguous pages and the pfn's that their
@@ -653,9 +673,22 @@ find_vmemmap_x86_64()
 
 	if (init_level4_pgt == NOT_FOUND_SYMBOL) {
 		ERRMSG("init_level4_pgt/init_top_pgt not found\n");
-		return FAILED;
+		return FALSE;
 	}
 
+	if (NUMBER(sme_mask) != NOT_FOUND_NUMBER)
+		pmask &= ~(NUMBER(sme_mask));
+
+	/*
+	 * vmemmap region can be randomized by KASLR.
+	 * (currently we don't utilize info->vmemmap_end on x86_64.)
+	 */
+	if (info->mem_map_data &&
+	    info->mem_map_data[0].mem_map != NOT_MEMMAP_ADDR)
+		info->vmemmap_start = info->mem_map_data[0].mem_map;
+
+	DEBUG_MSG("vmemmap_start: %16lx\n", info->vmemmap_start);
+
 	pagestructsize = size_table.page;
 	hugepagesize = PTRS_PER_PMD * info->page_size;
 	vaddr_base = info->vmemmap_start;
@@ -682,31 +715,31 @@ find_vmemmap_x86_64()
 		if (!readmem(PADDR, (unsigned long long)pgdp, (void *)&pud_addr,
 								sizeof(unsigned long))) {
 			ERRMSG("Can't get pgd entry for slot %d.\n", pgd_index);
-			return FAILED;
+			return FALSE;
 		}
 
 		/* mask the pgd entry for the address of the pud page */
-		pud_addr &= PMASK;
+		pud_addr &= pmask;
 		if (pud_addr == 0)
 			  continue;
 		/* read the entire pud page */
 		if (!readmem(PADDR, (unsigned long long)pud_addr, (void *)pud_page,
 					PTRS_PER_PUD * sizeof(unsigned long))) {
 			ERRMSG("Can't get pud entry for pgd slot %ld.\n", pgdindex);
-			return FAILED;
+			return FALSE;
 		}
 		/* step thru each pmd address in the pud page */
 		/* pudp points to an entry in the pud page */
 		for (pudp = (unsigned long *)pud_page, pudindex = 0;
 					pudindex < PTRS_PER_PUD; pudindex++, pudp++) {
-			pmd_addr = *pudp & PMASK;
+			pmd_addr = *pudp & pmask;
 			/* read the entire pmd page */
 			if (pmd_addr == 0)
 				continue;
 			if (!readmem(PADDR, pmd_addr, (void *)pmd_page,
 					PTRS_PER_PMD * sizeof(unsigned long))) {
 				ERRMSG("Can't get pud entry for slot %ld.\n", pudindex);
-				return FAILED;
+				return FALSE;
 			}
 			/* pmdp points to an entry in the pmd */
 			for (pmdp = (unsigned long *)pmd_page, pmdindex = 0;
@@ -741,7 +774,7 @@ find_vmemmap_x86_64()
 				 * - we discontiguous page is a string of valids
 				 */
 				if (pmd) {
-					data_addr = (pmd & PMASK);
+					data_addr = (pmd & pmask);
 					if (start_range) {
 						/* first-time kludge */
 						start_data_addr = data_addr;
@@ -782,7 +815,7 @@ find_vmemmap_x86_64()
 					num_pmds_valid++;
 					if (!(pmd & _PAGE_PSE)) {
 						printf("vmemmap pmd not huge, abort\n");
-						return FAILED;
+						return FALSE;
 					}
 				} else {
 					if (last_valid) {
@@ -914,7 +947,7 @@ find_vmemmap_x86_64()
 		i++;
 	} while (cur != vmaphead);
 	nr_gvmem_pfns = i;
-	return COMPLETED;
+	return TRUE;
 }
 
 #endif /* x86_64 */
