@@ -19,6 +19,8 @@
 #define TRUE		(1)
 #define FALSE		(0)
 #define ERROR		(-1)
+#define UNUSED   	(-1)
+#define RETURN_ON_ERROR  	(0x2)
 
 #ifndef LONG_MAX
 #define LONG_MAX	((long)(~0UL>>1))
@@ -35,12 +37,18 @@
 #define round(x, y)	(((x) / (y)) * (y))
 #define roundup(x, y)	((((x) + ((y) - 1)) / (y)) * (y))
 
+#define NUM_HEX  (0x1)
+#define NUM_DEC  (0x2)
+#define NUM_EXPR (0x4)
+#define NUM_ANY  (NUM_HEX|NUM_DEC|NUM_EXPR)
+
 /*
  * Incorrect address
  */
 #define NOT_MEMMAP_ADDR	(0x0)
 #define NOT_KV_ADDR	(0x0)
 #define NOT_PADDR	(ULONGLONG_MAX)
+#define BADADDR  	((ulong)(-1))
 
 #endif  /* COMMON_H */
 
